#include <stdio.h> 
#include <string.h>
#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>

#include "../mcu/serial.h"
#include "../iinchip/socket.h"
#include "../util/myprintf.h"

#include "timer.h"

#define TCNT0_VALUE	0xD7		// 5ms at 8MHz
#define TCNT2_VALUE	0x87		// 15ms at 8MHz

int sys_tick_5ms, sys_tick_1s;		// Tick count


/* Timer handler table */
struct _TIMER_IRQS
{
	TIMER_IRQ timer_irq[MAX_TIMER_CNT];
}TIMER_IRQS;


/*
********************************************************************************
*              TIMER ISR
*
* Description : This function counts timer tick value.
*   
* Arguments   : None
* Returns     : None
* Note        : Internal Function
********************************************************************************
*/
ISR(TIMER0_OVF_vect)		/* Interrupt Service Routine for AVR GCC ver. 3.4.6*/
//void timer0_irq(void)
{
	u_char i;
	sys_tick_5ms++;
	for(i=0; i < MAX_TIMER0_CNT; i++)
	{
		if(TIMER_IRQS.timer_irq[i].user_timer_handler)
			(*TIMER_IRQS.timer_irq[i].user_timer_handler)();
	}

	TCNT0 = TCNT0_VALUE;			/* reset TCNT0(5 ms) */
}

ISR(TIMER2_OVF_vect)		/* Interrupt Service Routine for AVR GCC ver. 3.4.6*/
//void timer2_irq(void)		/* signal handler for tcnt0 overflow interrupt */
{
	u_char i;
	
	if (++sys_tick_1s > 64) /* 1 sec. */
	{	
		sys_tick_1s = 0;
		for(i=MAX_TIMER0_CNT; i < MAX_TIMER_CNT; i++)
		{
			if(TIMER_IRQS.timer_irq[i].user_timer_handler)
				(*TIMER_IRQS.timer_irq[i].user_timer_handler)();
		}
	}
	TCNT2 = TCNT2_VALUE;		/* reset TCNT0(15 ms) */
}

/*
********************************************************************************
*              INIT. TIMER
*
* Description : This function initialize AVR timer.
*   
* Arguments   : 
* Returns     : None
* Note        :
********************************************************************************
*/
void init_timer(void)
{
	u_char i;
	cli();
	for(i=0;i<MAX_TIMER_CNT; i++)
		TIMER_IRQS.timer_irq[i].user_timer_handler = 0;
	TIMSK &= ~(1 << TOIE2);	
	TCNT2 = TCNT2_VALUE;		/* reset TCNT0(15 ms) */
	TCCR2 = 5;		/* count with cpu clock/1024 */
	sys_tick_1s = 0;

	TIMSK &= ~(1 << TOIE0);		/* disable TCNT0 overflow */
	TCNT0 = TCNT0_VALUE;			/* reset TCNT0(5 ms) */
	TCCR0 = 7;			/* count with cpu clock/1024 */		
	sys_tick_5ms = 0;
	sei();
}

/*
********************************************************************************
*              Regiser TIMER Handler
*
* Description : Register the timer handler
*   
* Arguments   : timer - Timer Handler Number
*		handler - user specific function to be called by timer interrupt
* Returns     : None
* Note        :
********************************************************************************
*/
void set_timer(u_int timer, void (*handler)(void)) 
{

	cli();
	if(timer < MAX_TIMER0_CNT)
		TIMSK |= (1 << TOIE0);
	else
		TIMSK |= (1 << TOIE2);
	if(handler) TIMER_IRQS.timer_irq[timer].user_timer_handler = handler;
	sei();
}


/*
********************************************************************************
*              Unregister Timer Handler
*
* Description : Unregister the timer handler
*   
* Arguments   : timer - user specific function to be called by timer interrupt
* Returns     : None
* Note        :
********************************************************************************
*/
void kill_timer(u_int timer) 
{
	u_char i;
	u_char bkill;
	cli();
	TIMER_IRQS.timer_irq[timer].user_timer_handler = 0;

	if(timer < MAX_TIMER0_CNT)
	{
		bkill = 1;
		for(i = 0; i < MAX_TIMER0_CNT; i++)
		{
			if(TIMER_IRQS.timer_irq[i].user_timer_handler)
			{
				bkill = 0;
				break;
			}
		}
		if(bkill) 
		{
			sys_tick_5ms = 0;
			TIMSK &= ~(1<<TOIE0);		
		}
	}
	else
	{
		bkill = 1;
		for(i = MAX_TIMER0_CNT; i < MAX_TIMER_CNT; i++)
		{
			if(TIMER_IRQS.timer_irq[i].user_timer_handler)
			{
				bkill = 0;
				break;
			}
		}
		if(bkill)
		{
			TIMSK &= ~(1<<TOIE2);
			sys_tick_1s = 0;
		}
	}
	sei();
}
